@@ -7,8 +7,8 @@
 //!use http_req::request;
 //!
 //!fn main() {
-//!    let mut buff = Vec::new();
-//!    let res = request::get("https://doc.rust-lang.org/", &mut buff).unwrap();
+//!    let mut writer = Vec::new(); //container for body of a response
+//!    let res = request::get("https://doc.rust-lang.org/", &mut writer).unwrap();
 //!
 //!    println!("Status: {} {}", res.status_code(), res.reason());
 //!}
@@ -16,4 +16,4 @@
 pub mod error;
 pub mod request;
 pub mod response;
-pub mod url;
+pub mod uri;
